@@ -266,6 +266,31 @@ class OllamaClient(BaseModelClient):
         last_error = None
         self._active_stream_session = None  # Track the active session
         
+        # First check if the model exists in our available models
+        try:
+            available_models = await self.get_available_models()
+            model_exists = False
+            available_model_names = []
+            
+            for m in available_models:
+                model_id = m.get("id", "")
+                available_model_names.append(model_id)
+                if model_id == model:
+                    model_exists = True
+                    break
+                    
+            if not model_exists:
+                debug_log(f"Model '{model}' not found in available models")
+                # Instead of failing, yield a helpful error message
+                yield f"Model '{model}' not found. Available models include: {', '.join(available_model_names[:5])}"
+                if len(available_model_names) > 5:
+                    yield f" and {len(available_model_names) - 5} more."
+                yield "\n\nPlease try a different model or check your spelling."
+                return
+        except Exception as e:
+            debug_log(f"Error checking model availability: {str(e)}")
+            # Continue anyway, the main request will handle errors
+        
         while retries >= 0:
             try:
                 # First try a quick test request to check if model is loaded
@@ -299,6 +324,16 @@ class OllamaClient(BaseModelClient):
                             if response.status != 200:
                                 logger.warning(f"Model test request failed with status {response.status}")
                                 debug_log(f"Model test request failed with status {response.status}")
+                                
+                                # Check if this is a 404 Not Found error
+                                if response.status == 404:
+                                    error_text = await response.text()
+                                    debug_log(f"404 error details: {error_text}")
+                                    # This is likely a model not found error
+                                    yield f"Error: Model '{model}' not found on the Ollama server."
+                                    yield "\nPlease check if the model name is correct or try pulling it first."
+                                    return
+                                    
                                 raise aiohttp.ClientError("Model not ready")
                     except (aiohttp.ClientError, asyncio.TimeoutError) as e:
                         logger.info(f"Model cold start detected: {str(e)}")
@@ -326,6 +361,16 @@ class OllamaClient(BaseModelClient):
                                 logger.error("Failed to pull model")
                                 debug_log("Failed to pull model")
                                 self._model_loading = False  # Reset flag on failure
+                                
+                                # Check if this is a 404 Not Found error
+                                if pull_response.status == 404:
+                                    error_text = await pull_response.text()
+                                    debug_log(f"404 error details: {error_text}")
+                                    # This is likely a model not found in registry
+                                    yield f"Error: Model '{model}' not found in the Ollama registry."
+                                    yield "\nPlease check if the model name is correct or try a different model."
+                                    return
+                                    
                                 raise Exception("Failed to pull model")
                             logger.info("Model pulled successfully")
                             debug_log("Model pulled successfully")
