@@ -707,10 +707,18 @@ class SimpleChatApp(App): # Keep SimpleChatApp class definition
                         else:
                             raise Exception("No valid API clients available for title generation")
 
-                # Generate title
+                # Generate title - make sure we're using the right client for the model
                 print(f"Calling generate_conversation_title with model: {model}")
                 log(f"Calling generate_conversation_title with model: {model}")
-                debug_log(f"Calling generate_conversation_title with model: {model}")
+                debug_log(f"Calling generate_conversation_title with model: {model}, client type: {type(client).__name__}")
+                
+                # Double-check that we're using the right client for this model
+                expected_client_type = BaseModelClient.get_client_type_for_model(model)
+                if expected_client_type and not isinstance(client, expected_client_type):
+                    debug_log(f"Warning: Client type mismatch. Expected {expected_client_type.__name__}, got {type(client).__name__}")
+                    debug_log("Creating new client with correct type")
+                    client = await BaseModelClient.get_client_for_model(model)
+                
                 title = await generate_conversation_title(content, model, client)
                 debug_log(f"Generated title: {title}")
                 log(f"Generated title: {title}")
@@ -729,11 +737,9 @@ class SimpleChatApp(App): # Keep SimpleChatApp class definition
                 # Update conversation object
                 self.current_conversation.title = title
                 
-                # IMPORTANT: Save the successful model for consistency
-                # If the title was generated with a different model than initially selected,
-                # update the selected_model to match so the response uses the same model
-                debug_log(f"Using same model for chat response: '{model}'")
-                self.selected_model = model
+                # DO NOT update the selected model here - keep the user's original selection
+                # This was causing issues with model mixing
+                debug_log(f"Keeping original selected model: '{self.selected_model}'")
 
                 self.notify(f"Conversation title set to: {title}", severity="information", timeout=3)
 
@@ -805,17 +811,23 @@ class SimpleChatApp(App): # Keep SimpleChatApp class definition
             style = self.selected_style
             
             debug_log(f"Using model: '{model}', style: '{style}'")
+            
+            # Determine the expected client type for this model
+            expected_client_type = BaseModelClient.get_client_type_for_model(model)
+            debug_log(f"Expected client type for {model}: {expected_client_type.__name__ if expected_client_type else 'None'}")
 
             # Ensure we have a valid model
             if not model:
                 debug_log("Model is empty, selecting a default model")
-                # Same fallback logic as in autotitling - this ensures consistency
+                # Check which providers are available and select an appropriate default
                 if OPENAI_API_KEY:
                     model = "gpt-3.5-turbo"
-                    debug_log("Falling back to OpenAI gpt-3.5-turbo")
+                    expected_client_type = BaseModelClient.get_client_type_for_model(model)
+                    debug_log(f"Falling back to OpenAI gpt-3.5-turbo with client type {expected_client_type.__name__ if expected_client_type else 'None'}")
                 elif ANTHROPIC_API_KEY:
-                    model = "claude-instant-1.2"
-                    debug_log("Falling back to Anthropic claude-instant-1.2")
+                    model = "claude-3-haiku-20240307"  # Updated to newer Claude model
+                    expected_client_type = BaseModelClient.get_client_type_for_model(model)
+                    debug_log(f"Falling back to Anthropic Claude 3 Haiku with client type {expected_client_type.__name__ if expected_client_type else 'None'}")
                 else:
                     # Check for a common Ollama model
                     try:
@@ -826,11 +838,13 @@ class SimpleChatApp(App): # Keep SimpleChatApp class definition
                             model = models[0].get("id", "llama3")
                         else:
                             model = "llama3"  # Common default
-                        debug_log(f"Falling back to Ollama model: {model}")
+                        expected_client_type = BaseModelClient.get_client_type_for_model(model)
+                        debug_log(f"Falling back to Ollama model: {model} with client type {expected_client_type.__name__ if expected_client_type else 'None'}")
                     except Exception as ollama_err:
                         debug_log(f"Error getting Ollama models: {str(ollama_err)}")
                         model = "llama3"  # Final fallback
-                        debug_log("Final fallback to llama3")
+                        expected_client_type = BaseModelClient.get_client_type_for_model(model)
+                        debug_log(f"Final fallback to llama3 with client type {expected_client_type.__name__ if expected_client_type else 'None'}")
 
             # Convert messages to API format with enhanced error checking
             api_messages = []
@@ -910,86 +924,61 @@ class SimpleChatApp(App): # Keep SimpleChatApp class definition
             message_display = MessageDisplay(assistant_message, highlight_code=CONFIG["highlight_code"])
             messages_container.mount(message_display)
             
-            # Force a layout refresh and scroll to end
+            # Force multiple layout refreshes and scroll to end to ensure visibility
+            self.refresh(layout=False)
+            await asyncio.sleep(0.01)
             self.refresh(layout=True)
             messages_container.scroll_end(animate=False)
+            await asyncio.sleep(0.01)
+            self.refresh(layout=True)
             
             # Add small delay to show thinking state
-            await asyncio.sleep(0.5)
+            await asyncio.sleep(0.3)
 
             # Stream chunks to the UI with synchronization
             update_lock = asyncio.Lock()
             last_refresh_time = time.time()  # Initialize refresh throttling timer
 
             async def update_ui(content: str):
-                # This function remains the same, called by the worker
+                # This function is called by the worker with each content update
                 if not self.is_generating:
                     debug_log("update_ui called but is_generating is False, returning.")
                     return
-
-                # Make last_refresh_time accessible in inner scope
-                nonlocal last_refresh_time
                 
                 async with update_lock:
                     try:
+                        # Add more verbose logging
+                        debug_log(f"update_ui called with content length: {len(content)}")
+                        print(f"update_ui: Updating with content length {len(content)}")
+                        
                         # Clear thinking indicator on first content
                         if assistant_message.content == "Thinking...":
                             debug_log("First content received, clearing 'Thinking...'")
-                            assistant_message.content = ""
-
+                            print("First content received, clearing 'Thinking...'")
+                            # We'll let the MessageDisplay.update_content handle this special case
+                        
                         # Update the message object with the full content
                         assistant_message.content = content
 
-                        # Update UI with the content - the MessageDisplay will now handle its own refresh
-                        # This is a critical change that ensures content is immediately visible
+                        # Update UI with the content - this now has special handling for "Thinking..."
+                        debug_log("Calling message_display.update_content")
                         await message_display.update_content(content)
                         
-                        # CRITICAL: Force immediate UI refresh after EVERY update 
-                        # This ensures we don't need a second Enter press to see content
+                        # More aggressive UI refresh sequence
+                        debug_log("Performing UI refresh sequence")
+                        # First do a lightweight refresh
+                        self.refresh(layout=False)
+                        # Then scroll to end
+                        messages_container.scroll_end(animate=False)
+                        # Then do a full layout refresh
                         self.refresh(layout=True)
-                        
-                        # Always scroll after each update to ensure visibility
+                        # Final scroll to ensure visibility
                         messages_container.scroll_end(animate=False)
                         
-                        # For longer responses, we can throttle the heavy refreshes
-                        # to reduce visual jitter, but still do light refreshes for every update
-                        content_length = len(content)
-                        
-                        # Define key refresh points that require more thorough updates
-                        new_paragraph = content.endswith("\n") and content.count("\n") > 0
-                        code_block = "```" in content
-                        needs_thorough_refresh = (
-                            content_length < 30 or       # Very aggressive for short responses
-                            content_length % 16 == 0 or  # More frequent periodic updates 
-                            new_paragraph or             # Refresh on paragraph breaks
-                            code_block                   # Refresh when code blocks are detected
-                        )
-                        
-                        # Check if it's been enough time since last heavy refresh
-                        # Reduced from 200ms to 100ms for more responsive UI
-                        current_time = time.time()
-                        time_since_refresh = current_time - last_refresh_time
-                        
-                        if needs_thorough_refresh and time_since_refresh > 0.1:
-                            # Store the time we did the heavy refresh
-                            last_refresh_time = current_time
-                            
-                            # Ensure content is visible with an aggressive, guaranteed update sequence
-                            # 1. Scroll to ensure visibility
-                            messages_container.scroll_end(animate=False)
-                            
-                            # 2. Force a comprehensive refresh with layout recalculation
-                            self.refresh(layout=True)
-                            
-                            # 3. Small delay for rendering
-                            await asyncio.sleep(0.01)
-                            
-                            # 4. Another scroll to account for any layout changes
-                            messages_container.scroll_end(animate=False)
-                            
                     except Exception as e:
                         debug_log(f"Error updating UI: {str(e)}")
                         log.error(f"Error updating UI: {str(e)}")
+                        print(f"Error updating UI: {str(e)}")
 
             # --- Remove the inner run_generation_worker function ---
 
@@ -1053,14 +1042,32 @@ class SimpleChatApp(App): # Keep SimpleChatApp class definition
                 error = worker.error
                 debug_log(f"Error in generation worker: {error}")
                 log.error(f"Error in generation worker: {error}")
-                self.notify(f"Generation error: {error}", severity="error", timeout=5)
+                
+                # Sanitize error message for UI display
+                error_str = str(error)
+                
+                # Check if this is an Ollama error
+                is_ollama_error = "ollama" in error_str.lower() or "404" in error_str
+                
+                # Create a user-friendly error message
+                if is_ollama_error:
+                    # For Ollama errors, provide a more user-friendly message
+                    user_error = "Unable to generate response. The selected model may not be available."
+                    debug_log(f"Sanitizing Ollama error to user-friendly message: {user_error}")
+                    # Show technical details only in notification, not in chat
+                    self.notify(f"Model error: {error_str}", severity="error", timeout=5)
+                else:
+                    # For other errors, show a generic message
+                    user_error = f"Error generating response: {error_str}"
+                    self.notify(f"Generation error: {error_str}", severity="error", timeout=5)
+                
                 # Add error message to UI
                 if self.messages and self.messages[-1].role == "assistant":
                     debug_log("Removing thinking message")
                     self.messages.pop()  # Remove thinking message
-                error_msg = f"Error: {error}"
-                debug_log(f"Adding error message: {error_msg}")
-                self.messages.append(Message(role="assistant", content=error_msg))
+                
+                debug_log(f"Adding error message: {user_error}")
+                self.messages.append(Message(role="assistant", content=user_error))
                 await self.update_messages_ui()
 
             elif worker.state == "success":
